#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "matrix.h"
#include "view.h"
#include "display.h"
#include "object.h"
#include "main.h"

#define FRUSTRUM        55.0
#define FRONT           500.0
#define DEPTH           1000.0
#define BACK            -3000.0



/* basic file reading thing... expand this later... */
void            get_parameters(filename, vrp, theta)
char           *filename;
point          *vrp;
double         *theta;
{
   FILE           *fptr;
   char            readstr[100];
   float           f1, f2, f3;
   short           go;

   if (!(fptr = fopen(filename, "r")))
   {
      fprintf(stderr, "Cannot open parameter file %s.\n", filename);
      exit(1);
   }
   go = 1;
   while (go)
   {
      if (!fgets(readstr, 100, fptr))
      {
         fprintf(stderr, "Error reading file %s.\n", filename);
         exit(1);
      }
      switch (readstr[0])
      {
         case 'v':
            fscanf(fptr, "%f %f %f", &f1, &f2, &f3);
            vrp->x = f1;
            vrp->y = f2;
            vrp->z = f3;
            break;
         case 't':
            fscanf(fptr, "%f", &f1);
            *theta = f1;
            break;
         case 'e':
            go = 0;
            break;
         default:
            /* skip */
            break;
      }
   }
   fclose(fptr);
}

void            main(argc, argv)
int             argc;
char           *argv[];
{
   double          theta, elev, forw;
   unsigned int    mask;
   float           xs, ys, zs;
   vector          temp;
   point           vrp;
   int             xx, yy;

   if (argc < 4)
   {
      fprintf(stderr, "Usage: %s datafile x_scale y z\n", argv[0]);
      exit(1);
   }

   init_display(100, 100, VR, VB);      /* initialize client */

   get_parameters("input", &vrp, &theta);

   sscanf(argv[2], "%f", &xs);
   sscanf(argv[3], "%f", &ys);
   sscanf(argv[4], "%f", &zs);
   init_object(argv[1], xs, ys, zs);


   eval_projection(DEPTH, FRONT, BACK, FRUSTRUM);
   eval_viewport(VVL, VVR, VVT, VVB, 0.0, 1.0);

   forw = elev = theta = 0.0;
   eval_orientation(theta, elev, forw);
#ifdef TRANSFORM
   transform_object();
#endif
   draw_object();

   printf("theta= %f,forw= %f,elev= %f\n", theta, forw, elev);
   print_vrp();

   while (1)
   {
      if (get_pointer(&xx, &yy, &mask))
      {
         if ((mask == Button1Mask) || (mask == Button2Mask)
             || (mask == Button3Mask))
         {
            forw = elev = 0.0;
            switch (mask)
            {
               case Button1Mask:
                  theta -= (double) (xx - (VR / 2)) / 60.0;
                  if (theta > 360.0)
                     theta -= 360.0;
                  if (theta < 0.0)
                     theta += 360.0;
                  forw = (double) (yy - (VB / 2)) / 60.0;
                  break;

               case Button2Mask:
                  elev = -(double) (yy - (VB / 2)) / 60.0;
                  break;

               case Button3Mask:
                  get_parameters("input", &vrp, &theta);
                  init_orientation(&vrp, temp, temp);
                  break;
            }
            eval_orientation(theta, elev, forw);
#ifdef TRANSFORM
            transform_object();
#endif
            draw_object();

            printf("theta= %f,forw= %f,elev= %f\n", theta, forw, elev);
            print_vrp();
         }
      }
   }
   close_display();             /* close client */
}
